package com.stormtales.notevault.network.sync;

import android.content.Context;
import android.net.Uri;
import com.stormtales.notevault.data.entities.NoteSheet;
import com.stormtales.notevault.data.entities.Song;
import com.stormtales.notevault.data.repositories.SongRepository;
import com.stormtales.notevault.data.repositories.SongSyncRepository;
import com.stormtales.notevault.network.sync.models.*;
import com.stormtales.notevault.ui.gallery.GalleryViewModel;
import com.stormtales.notevault.utils.NoteSheetsUtil;

import java.io.File;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

public class SongSyncModule {
    private SongRepository songRepository;
    private SongSyncRepository songSyncRepository;
    private SongSyncService songSyncService;
    private GalleryViewModel syncViewModel;
    private Context context;

    public SongSyncModule(Context context, GalleryViewModel syncViewModel) {
        this.songRepository = new SongRepository(context);
        this.songSyncRepository = new SongSyncRepository(context);
        this.songSyncService = new SongSyncService(context);
        this.syncViewModel = syncViewModel;
        this.context = context;
    }

    public void syncCreatedSongs() {
        songSyncRepository.loadCreatedSongs(result -> {
            songSyncService.syncCreatedSongs(result, response -> {
                songSyncRepository.markCreatedSongsAsSynced(response);
                if(response.getCreateResponses().isEmpty()) {
                    syncViewModel.finishCreateSongSyncing();
                } else {
                    uploadCreatedNoteSheets(result, response);
                }
            });
        });
    }

    public void uploadCreatedNoteSheets(List<Song> result, BatchCreateResponse response) {
        List<Integer> songIDs = result.stream().map(Song::getLocalID).collect(Collectors.toList());
        songSyncRepository.getNoteSheetFilesBySongIDs(noteSheets -> {
            songSyncService.uploadNoteSheetsOfCreatedSongs(noteSheets, response, uploadResponses -> {
                songSyncRepository.markCreatedNoteSheetsAsSynced(uploadResponses);
                syncViewModel.finishCreateSongSyncing();
            });
        });
    }

    public void syncModifiedSongs() {
        songSyncRepository.loadModifiedSongs(result -> {
            songSyncService.syncModifiedSongs(result, (FinishSongSyncingCallback<SongModifyBatchResponse>) response -> {
                songSyncRepository.markModifiedSongsAsSynced(response);
                uploadModifiedNoteSheets();
            });
        });
    }

    public void uploadModifiedNoteSheets() {
        songSyncRepository.loadModifiedNoteSheets(modifiedNoteSheets -> {
            songSyncService.uploadModifiedNoteSheets(modifiedNoteSheets, new SongSyncService.UploadNoteSheetCallback() {
                @Override
                public void finishUploadNoteSheets(List<UploadResponse> uploadResponses) {
                    songSyncRepository.markModifiedNoteSheetsAsSynced(uploadResponses);
                    syncViewModel.finishModifiedSongSyncinc();
                }
            });
        });
    }

    public void syncDeletedSongs() {
        songSyncRepository.loadDeletedSongs(result -> {
            songSyncService.syncDeletedSong(result, (remoteDeletedSongs, localDeletedSongs) -> {
                songSyncRepository.markDeletedSongsAsSynced(remoteDeletedSongs, localDeletedSongs);
                syncViewModel.finishDeleteSongSyncinc();
            });
        });
    }

    public void fetchRemoteModifiedSongs() {
        //Todo: Determine Last Client Sync; for testing use LocalDateTime.MIN
        songSyncService.fetchRemoteModifiedSongs(LocalDateTime.now().minusDays(35), new SongSyncRepository.LoadDataCallback<FetchResponse>() {
            @Override
            public void onResult(FetchResponse result) {
                songSyncRepository.markSongsAsRemotelyModified(result.getServerIDs());
                getRemotelyModifiedSongData();
            }
        });
    }

    public void getRemotelyModifiedSongData() {
        songSyncRepository.loadRemotelyModifiedSongs(songs -> {
            songSyncService.getRemotelyModifiedSongData(songs, remoteSongs -> {
                songSyncRepository.saveRemoteSongs(remoteSongs, this::downloadNoteSheets);

            });

        });
    }

    public void downloadNoteSheets(List<NoteSheet> noteSheets) {
        for(NoteSheet noteSheet : noteSheets) {
            songSyncService.downloadNoteSheet(noteSheet, responseBody -> {
                String localFilename = noteSheet.getServerFileName().substring(36);

                NoteSheetsUtil.saveImageFromServer(responseBody, context.getFilesDir());
            });
        }
        syncViewModel.finishFetching();
    }

    public interface FinishSongCreateSyncingCallback {
        void finishSongSyncing(BatchCreateResponse response);
    }

    public interface FinishSongSyncingCallback<T> {
        void finishSongSyncing(T response);
    }
}
