package com.stormtales.notevault.network.sync;

import android.content.Context;
import android.util.Log;
import com.stormtales.notevault.data.entities.NoteSheet;
import com.stormtales.notevault.data.entities.Song;
import com.stormtales.notevault.data.repositories.SongSyncRepository;
import com.stormtales.notevault.network.NetworkModule;
import com.stormtales.notevault.network.auth.AuthAPI;
import com.stormtales.notevault.network.sync.models.*;
import com.stormtales.notevault.ui.gallery.GalleryViewModel;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.RequestBody;
import okhttp3.ResponseBody;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

import java.io.File;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class SongSyncService {
    private final SongSyncAPI songSyncAPI;
    private GalleryViewModel syncStatusViewModel;

    public SongSyncService(Context context) {
        songSyncAPI = NetworkModule.getRetrofitInstance(context).create(SongSyncAPI.class);
    }

    public void syncCreatedSongs(List<Song> songs, SongSyncModule.FinishSongCreateSyncingCallback callback) {
        List<SongCreateRequest> createRequests = new ArrayList<>();
        for(Song song : songs) {
            createRequests.add(new SongCreateRequest(song));
        }
        SongCreateBatchRequest songCreateBatchRequest = new SongCreateBatchRequest(createRequests);
        songSyncAPI.syncCreatedSongs(songCreateBatchRequest).enqueue(new Callback<BatchCreateResponse>() {
            @Override
            public void onResponse(Call<BatchCreateResponse> call, Response<BatchCreateResponse> response) {
                callback.finishSongSyncing(response.body());
            }

            @Override
            public void onFailure(Call<BatchCreateResponse> call, Throwable throwable) {

            }
        });
    }

    public void uploadNoteSheetsOfCreatedSongs(List<NoteSheet> noteSheets, BatchCreateResponse batchCreateResponse, UploadNoteSheetCallback callback) {
        List<UploadResponse> uploadResponses = new ArrayList<>();
        for(CreateResponse createResponse : batchCreateResponse.getCreateResponses()) {
            for(NoteSheet noteSheet : noteSheets) {
                if(noteSheet.getSongID() == createResponse.getLocalID()) {
                    File imageFile = new File(noteSheet.getLocalFileName());

                    RequestBody serverID = RequestBody.create(MediaType.parse("text/plain"), createResponse.getServerID());
                    RequestBody fileName = RequestBody.create(MediaType.parse("text/plain"), imageFile.getName());

                    RequestBody requestFile = RequestBody.create(MediaType.parse("image/**"), new File(noteSheet.getLocalFileName()));
                    MultipartBody.Part image = MultipartBody.Part.createFormData("image", noteSheet.getLocalFileName(), requestFile);

                    songSyncAPI.uploadNoteSheet(serverID, fileName, image).enqueue(new Callback<UploadResponse>() {
                        @Override
                        public void onResponse(Call<UploadResponse> call, Response<UploadResponse> response) {
                            if(response.isSuccessful() && response.body() != null) {
                                uploadResponses.add(response.body());
                                if(uploadResponses.size() == noteSheets.size()) {
                                    callback.finishUploadNoteSheets(uploadResponses);
                                }
                            }
                        }

                        @Override
                        public void onFailure(Call<UploadResponse> call, Throwable throwable) {

                        }
                    });
                }
            }
        }
    }

    public void syncModifiedSongs(Map<Song, List<NoteSheet>> songs, SongSyncModule.FinishSongSyncingCallback<SongModifyBatchResponse> callback) {
        List<SongModifyRequest> modifyRequests = new ArrayList<>();
        for(Map.Entry<Song, List<NoteSheet>> entry: songs.entrySet()) {
            modifyRequests.add(new SongModifyRequest(entry.getKey(), entry.getValue()));
        }
        SongModifyBatchRequest songModifyBatchRequest = new SongModifyBatchRequest(modifyRequests);
        songSyncAPI.syncModifiedSongs(songModifyBatchRequest).enqueue(new Callback<SongModifyBatchResponse>() {
            @Override
            public void onResponse(Call<SongModifyBatchResponse> call, Response<SongModifyBatchResponse> response) {
                callback.finishSongSyncing(response.body());
            }

            @Override
            public void onFailure(Call<SongModifyBatchResponse> call, Throwable throwable) {

            }
        });
    }

    public void syncDeletedSong(List<Song> songs, SyncDeletedSongsCallback callback) {
        List<String> deleteRequests = new ArrayList<>();
        List<Integer> deletedNotSyncedSongs = new ArrayList<>();
        for(Song song : songs) {
            if(song.getServerID() == null) {
                deletedNotSyncedSongs.add(song.getLocalID());
            } else {
                deleteRequests.add(song.getServerID());
            }
        }

        SongBatchDeleteRequest songBatchDeleteRequest = new SongBatchDeleteRequest(deleteRequests);
        songSyncAPI.syncDeletedSongs(songBatchDeleteRequest).enqueue(new Callback<BatchModifyResponse>() {
            @Override
            public void onResponse(Call<BatchModifyResponse> call, Response<BatchModifyResponse> response) {
                if(response.isSuccessful() && response.body() != null) {
                    callback.finishSongSyncing(response.body().getModifiedServerObjects(), deletedNotSyncedSongs);
                }
            }

            @Override
            public void onFailure(Call<BatchModifyResponse> call, Throwable throwable) {

            }
        });
    }

    public void uploadModifiedNoteSheets(List<NoteSheet> modifiedNoteSheets, UploadNoteSheetCallback callback) {
        List<UploadResponse> uploadResponses = new ArrayList<>();
        for(NoteSheet noteSheet : modifiedNoteSheets) {
            RequestBody server_filename = RequestBody.create(MediaType.parse("text/plain"), noteSheet.getServerFileName());

            RequestBody requestFile = RequestBody.create(MediaType.parse("image/**"), new File(noteSheet.getLocalFileName()));
            MultipartBody.Part image = MultipartBody.Part.createFormData("image", noteSheet.getLocalFileName(), requestFile);

            songSyncAPI.updateNoteSheet(server_filename, image).enqueue(new Callback<UploadResponse>() {
                @Override
                public void onResponse(Call<UploadResponse> call, Response<UploadResponse> response) {
                    if(response.isSuccessful() && response.body() != null) {
                        uploadResponses.add(response.body());
                        if(uploadResponses.size() == modifiedNoteSheets.size()) {
                            callback.finishUploadNoteSheets(uploadResponses);
                        }
                    } else {
                        Log.d("SongSyncService", "Something went wrong");
                    }
                }

                @Override
                public void onFailure(Call<UploadResponse> call, Throwable throwable) {
                    Log.e("SongSyncService", "Upload failed: " + throwable.getMessage(), throwable);
                }
            });
        }
    }

    public void fetchRemoteModifiedSongs(LocalDateTime lastClientSync, SongSyncRepository.LoadDataCallback<FetchResponse> callback) {
        String formattedTime = lastClientSync.format(DateTimeFormatter.ISO_LOCAL_DATE_TIME);
        songSyncAPI.fetchRemoteModifiedSongs(formattedTime).enqueue(new Callback<FetchResponse>() {
            @Override
            public void onResponse(Call<FetchResponse> call, Response<FetchResponse> response) {
                if(response.isSuccessful() && response.body() != null) {
                    callback.onResult(response.body());
                }
            }

            @Override
            public void onFailure(Call<FetchResponse> call, Throwable throwable) {
                Log.d("SongSyncService", "Fetch failed: " + throwable.getMessage(), throwable);
            }
        });
    }

    public void getRemotelyModifiedSongData(List<Song> remotelyModifiedSongs, SongSyncRepository.LoadDataCallback<List<SongModel>> callback) {
        List<SongModel> songModels = new ArrayList<>();
        for(Song song : remotelyModifiedSongs) {
            songSyncAPI.fetchRemotelyModifiedSongData(song.getServerID()).enqueue(new Callback<SongModel>() {
                @Override
                public void onResponse(Call<SongModel> call, Response<SongModel> response) {
                    if(response.isSuccessful() && response.body() != null) {
                        songModels.add(response.body());
                        if(songModels.size() == remotelyModifiedSongs.size()) {
                            callback.onResult(songModels);
                        }
                    }
                }

                @Override
                public void onFailure(Call<SongModel> call, Throwable throwable) {
                    Log.d("SongSyncService", "Fetch failed: " + throwable.getMessage(), throwable);
                }
            });
        }
    }

    public void downloadNoteSheet(NoteSheet noteSheet, SongSyncRepository.LoadDataCallback<ResponseBody> callback) {
        songSyncAPI.downloadNotesheet(noteSheet.getServerFileName()).enqueue(new Callback<ResponseBody>() {
            @Override
            public void onResponse(Call<ResponseBody> call, Response<ResponseBody> response) {
                if(response.isSuccessful() && response.body() != null) {
                    callback.onResult(response.body());
                }
            }

            @Override
            public void onFailure(Call<ResponseBody> call, Throwable throwable) {
                Log.d("SongSyncService", "Download failed: " + throwable.getMessage(), throwable);
            }
        });
    }


    public interface SyncDeletedSongsCallback {
        void finishSongSyncing(List<String> remoteDeletedSongs, List<Integer> localDeletedSongs);
    }

    public interface UploadNoteSheetCallback {
        void finishUploadNoteSheets(List<UploadResponse> uploadResponses);
    }
}
