package com.stormtales.notevault.utils;

import android.content.ContentResolver;
import android.content.Context;
import android.media.ExifInterface;
import android.net.Uri;
import okhttp3.ResponseBody;

import java.io.*;
import java.nio.file.Files;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

public class NoteSheetsUtil {
    private static long getImageTimestamp(Context context, Uri imageUri) {
        try  {
            InputStream inputStream = context.getContentResolver().openInputStream(imageUri);
            ExifInterface exif = new ExifInterface(inputStream);

            String dateTime = exif.getAttribute(ExifInterface.TAG_DATETIME);
            if(dateTime != null) {
                SimpleDateFormat format = new SimpleDateFormat("yyyy:MM:dd HH:mm:ss", Locale.getDefault());
                Date date = format.parse(dateTime);
                return date != null ? date.getTime() : 0;
            } else {
                File file = new File(imageUri.getPath());
                return file.lastModified();
            }
        } catch (IOException | ParseException e) {
            throw new RuntimeException(e);
        }
    }

    public static void sortNoteSheetsByTimestamp(Context context, Uri[] uris) {
        ArrayList<UriTimestamp> uriTimestamps = new ArrayList<>();

        for(Uri uri : uris) {
            long timestamp = getImageTimestamp(context, uri);
            uriTimestamps.add(new UriTimestamp(uri, timestamp));
        }

        Collections.sort(uriTimestamps, Comparator.comparingLong(UriTimestamp::getTimestamp));

        for(int i=0; i<uriTimestamps.size(); i++) {
            uris[i] = uriTimestamps.get(i).getUri();
        }
    }

    public static Tupel<String, String> saveImageInternally(ContentResolver contentResolver, Uri uri, File filesDir) {
        try (InputStream inputStream = contentResolver.openInputStream(uri)) {
            // Datei erstellen
            File imageFile = new File(filesDir, "saved_image_" + System.currentTimeMillis() + ".jpg");
            try (OutputStream outputStream = Files.newOutputStream(imageFile.toPath())) {
                // SHA-256 Hash-Funktion initialisieren
                MessageDigest digest = MessageDigest.getInstance("SHA-256");

                // Daten blockweise lesen und gleichzeitig schreiben und hashen
                byte[] buffer = new byte[4096];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                    digest.update(buffer, 0, bytesRead); // Hash aktualisieren
                }

                // Hash berechnen
                byte[] hashBytes = digest.digest();
                StringBuilder hashString = new StringBuilder();
                for (byte b : hashBytes) {
                    hashString.append(String.format("%02x", b)); // Bytes in Hexadezimal umwandeln
                }

                // Hashwert zurückgeben (kann auch zusammen mit dem Pfad gespeichert werden)
                return new Tupel<>(imageFile.getAbsolutePath(), hashString.toString());
            } catch (NoSuchAlgorithmException e) {
                throw new RuntimeException(e);
            }
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    public static Tupel<String, String> saveImageFromServer(ResponseBody body, File filesDir) {
        File imageFile = new File(filesDir, "saved_image_" + System.currentTimeMillis() + ".jpg");
        try (InputStream inputStream = body.byteStream();
             FileOutputStream outputStream = new FileOutputStream(imageFile)) {
            MessageDigest digest = MessageDigest.getInstance("SHA-256");

            // Daten blockweise lesen und gleichzeitig schreiben und hashen
            byte[] buffer = new byte[4096];
            int bytesRead;
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
                digest.update(buffer, 0, bytesRead); // Hash aktualisieren
            }

            // Hash berechnen
            byte[] hashBytes = digest.digest();
            StringBuilder hashString = new StringBuilder();
            for (byte b : hashBytes) {
                hashString.append(String.format("%02x", b)); // Bytes in Hexadezimal umwandeln
            }

            // Hashwert zurückgeben (kann auch zusammen mit dem Pfad gespeichert werden)
            return new Tupel<>(imageFile.getAbsolutePath(), hashString.toString());
        } catch (IOException | NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
    }

    public static void deleteNoteSheet(String filePath) throws IOException {
        Files.delete(new File(filePath).toPath());
    }
}
